@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -19,7 +19,6 @@
 #include "squid.h"
 #include "acl/Acl.h"
 #include "acl/AclDenyInfoList.h"
-#include "acl/AclNameList.h"
 #include "acl/Checklist.h"
 #include "acl/Gadgets.h"
 #include "acl/Strategised.h"
@@ -105,9 +104,8 @@ void
 aclParseDenyInfoLine(AclDenyInfoList ** head)
 {
     char *t = NULL;
-    AclDenyInfoList *A = NULL;
-    AclDenyInfoList *B = NULL;
-    AclDenyInfoList **T = NULL;
+    AclDenyInfoList *B;
+    AclDenyInfoList **T;
     AclNameList *L = NULL;
     AclNameList **Tail = NULL;
 
@@ -119,16 +117,13 @@ aclParseDenyInfoLine(AclDenyInfoList ** head)
         return;
     }
 
-    A = (AclDenyInfoList *)memAllocate(MEM_ACL_DENY_INFO_LIST);
-    A->err_page_id = errorReservePageId(t);
-    A->err_page_name = xstrdup(t);
-    A->next = (AclDenyInfoList *) NULL;
+    AclDenyInfoList *A = new AclDenyInfoList(t);
+
     /* next expect a list of ACL names */
     Tail = &A->acl_list;
 
     while ((t = ConfigParser::NextToken())) {
-        L = (AclNameList *)memAllocate(MEM_ACL_NAME_LIST);
-        xstrncpy(L->name, t, ACL_NAME_SZ-1);
+        L = new AclNameList(t);
         *Tail = L;
         Tail = &L->next;
     }
@@ -136,7 +131,7 @@ aclParseDenyInfoLine(AclDenyInfoList ** head)
     if (A->acl_list == NULL) {
         debugs(28, DBG_CRITICAL, "aclParseDenyInfoLine: " << cfg_filename << " line " << config_lineno << ": " << config_input_line);
         debugs(28, DBG_CRITICAL, "aclParseDenyInfoLine: deny_info line contains no ACL's, skipping");
-        memFree(A, MEM_ACL_DENY_INFO_LIST);
+        delete A;
         return;
     }
 
@@ -315,8 +310,7 @@ aclDestroyDenyInfoList(AclDenyInfoList ** list)
         }
 
         a_next = a->next;
-        xfree(a->err_page_name);
-        memFree(a, MEM_ACL_DENY_INFO_LIST);
+        delete a;
     }
 
     *list = NULL;
